/*
 * *************************************************
 * Copyright (c) 2018 QMTH. All Rights Reserved.
 * Created by Deason on 2018-07-31 17:19:06.
 * *************************************************
 */

package cn.com.qmth.examcloud.app.service.impl;

import cn.com.qmth.examcloud.api.commons.enums.ExamType;
import cn.com.qmth.examcloud.app.core.SysProperty;
import cn.com.qmth.examcloud.app.core.exception.ApiException;
import cn.com.qmth.examcloud.app.core.utils.DateUtils;
import cn.com.qmth.examcloud.app.core.utils.HttpUtils;
import cn.com.qmth.examcloud.app.model.Constants;
import cn.com.qmth.examcloud.app.model.GetYunSignatureReq;
import cn.com.qmth.examcloud.app.model.Result;
import cn.com.qmth.examcloud.app.service.CoreOeService;
import cn.com.qmth.examcloud.commons.util.JsonMapper;
import cn.com.qmth.examcloud.support.cache.CacheHelper;
import cn.com.qmth.examcloud.support.cache.bean.ExamPropertyCacheBean;
import cn.com.qmth.examcloud.support.cache.bean.ExamSettingsCacheBean;
import cn.com.qmth.examcloud.support.enums.ExamProperties;
import com.alibaba.fastjson.JSONObject;
import okhttp3.FormBody;
import okhttp3.FormBody.Builder;
import okhttp3.MediaType;
import okhttp3.MultipartBody;
import okhttp3.RequestBody;
import org.apache.commons.io.FilenameUtils;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.Assert;
import org.springframework.web.multipart.MultipartFile;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * 网考业务服务接口
 *
 * @author: QMTH
 * @since: 2018/7/16
 */
@Service
public class CoreOeServiceImpl implements CoreOeService {

    private static final Logger log = LoggerFactory.getLogger(CoreOeServiceImpl.class);

    @Autowired
    private SysProperty sysProperty;

    @Override
    public Result getCurrentTime(String key, String token) throws Exception {
        Result<String> result = new Result<>();
        result.success();
        result.setData(DateUtils.formatLongDate(String.valueOf(System.currentTimeMillis())));
        return result;
    }

    @Override
    public Result getOfflineExamCourseList(String key, String token) throws Exception {
        //封装请求参数									
        final String requestUrl = String.format("%s/api/branch_ecs_oe_admin/offlineExam/getOfflineCourse", sysProperty.getApiDomain());
        return HttpUtils.doGet(requestUrl, key, token);
    }

    @Override
    public Result startOfflineExamRecord(String key, String token, String examStudentId) throws Exception {
        //封装请求参数									
        final String requestUrl = String.format("%s/api/branch_ecs_oe_admin/offlineExam/startOfflineExam?examStudentId=%s", sysProperty.getApiDomain(), examStudentId);
        return HttpUtils.doGet(requestUrl, key, token);
    }

    @Override
    public Result uploadPaperAnswer(String key, String token, String examRecordId, byte[] fileBytes, String fileName, String md5) throws Exception {
        Assert.notNull(examRecordId, "FileName must not be null.");
        Assert.notNull(fileName, "ExamRecordId must not be null.");
        String fileType = FilenameUtils.getExtension(fileName.toLowerCase());
        if (!Constants.FILE_TYPE_ZIP.equals(fileType) && !Constants.FILE_TYPE_PDF.equals(fileType)
                && !Constants.FILE_TYPE_JPG.equals(fileType) && !Constants.FILE_TYPE_JPEG.equals(fileType)
                && !Constants.FILE_TYPE_PNG.equals(fileType)) {
            throw new ApiException("FileType must be zip,pdf,jpg,jpeg,png.");
        }
        if (fileBytes.length == 0) {
            throw new ApiException("File must be not empty.");
        }
        //封装请求参数									
        final String requestUrl = String.format("%s/api/branch_ecs_oe_admin/offlineExam/submitPaper?examRecordDataId=%s", sysProperty.getApiDomain(), examRecordId);
        MultipartBody.Builder form = new MultipartBody.Builder().setType(MultipartBody.FORM);
        RequestBody body = RequestBody.create(MediaType.parse("application/octet-stream"), fileBytes);
        form.addFormDataPart("file", fileName, body);
        form.addFormDataPart("fileType", fileType);
        form.addFormDataPart("md5", md5 != null ? md5 : "");
        return HttpUtils.doPost(requestUrl, form.build(), key, token);
    }

    @Override
    public Result getPracticeExamCourseList(String key, String token, String examId) throws Exception {
        //封装请求参数
        final String requestUrl = String.format("%s/api/branch_ecs_oe_admin/practice/queryPracticeCourseList?examId=%s", sysProperty.getApiDomain(), examId);
        return HttpUtils.doGet(requestUrl, key, token);
    }

    @Override
    public Result getExamRecordHeartbeat(String key, String token) throws Exception {
        log.warn("{} examHeartbeat...", key);
        //封装请求参数
        final String requestUrl = String.format("%s/api/ecs_oe_student/examControl/examHeartbeat", sysProperty.getApiDomain());
        return HttpUtils.doGet(requestUrl, key, token);
    }

    @Override
    public Result startPracticeExamRecord(String key, String token, String examStudentId) throws Exception {
        //封装请求参数
        final String requestUrl = String.format("%s/api/ecs_oe_student/examControl/startExam?examStudentId=%s", sysProperty.getApiDomain(), examStudentId);
        return HttpUtils.doGet(requestUrl, key, token);
    }

    @Override
    public Result switchScreen(String key, String token, String examRecordDataId) throws Exception {
        //封装请求参数
        final String requestUrl = String.format("%s/api/ecs_oe_student/examControl/switchScreen?examRecordDataId=%s", sysProperty.getApiDomain(), examRecordDataId);
        RequestBody formBody = FormBody.create(MediaType.parse(Constants.CHARSET_JSON_UTF8), "");
        return HttpUtils.doPost(requestUrl, formBody, key, token);
    }

    @Override
    public Result getAdminExamPaperStructList(String key, String token, String examRecordId, String fromCache) throws Exception {
        //封装请求参数
        String requestUrl = null;
        if ("1".equals(fromCache)) {
            requestUrl = String.format("%s/api/branch_ecs_oe_admin/examRecordPaperStruct/getExamRecordPaperStruct?fromCache=1&examRecordDataId=%s", sysProperty.getApiDomain(), examRecordId);
        } else {
            requestUrl = String.format("%s/api/branch_ecs_oe_admin/examRecordPaperStruct/getExamRecordPaperStruct?examRecordDataId=%s", sysProperty.getApiDomain(), examRecordId);
        }
        return HttpUtils.doGet(requestUrl, key, token);
    }

    @Override
    public Result getStudentExamPaperStructList(String key, String token, String examRecordId) throws Exception {
        //封装请求参数
        String requestUrl = String.format("%s/api/ecs_oe_student/examRecordPaperStruct/getExamRecordPaperStruct?fromCache=1&examRecordDataId=%s", sysProperty.getApiDomain(), examRecordId);
        return HttpUtils.doGet(requestUrl, key, token);
    }

    @Override
    public Result getExamRecordPaperQuestionList(String key, String token) throws Exception {
        //封装请求参数
        final String requestUrl = String.format("%s/api/ecs_oe_student/examQuestion/findExamQuestionList", sysProperty.getApiDomain());
        return HttpUtils.doGet(requestUrl, key, token);
    }

    @Override
    public Result getExamRecordPaperQuestionDetail(String key, String token, String questionId, String examRecordId) throws Exception {
        //封装请求参数
        final String requestUrl = String.format("%s/api/ecs_oe_student/examQuestion/getQuestionContent?questionId=%s&exam_record_id=%s", sysProperty.getApiDomain(), questionId, examRecordId);
        return HttpUtils.doGet(requestUrl, key, token);
    }

    @Override
    public Result updateExamRecordPaperQuestionAnswer(String key, String token, Integer order, String studentAnswer, String audioPlayTimes) throws Exception {
        //封装请求参数
        final String requestUrl = String.format("%s/api/ecs_oe_student/examQuestion/submitQuestionAnswer", sysProperty.getApiDomain());
        List<Map> paramsList = new ArrayList<Map>();
        Map<String, Object> params = new HashMap<>();
        params.put("order", order);
        params.put("studentAnswer", studentAnswer);
        params.put("audioPlayTimes", audioPlayTimes);
        paramsList.add(params);
        String json = new JsonMapper().toJson(paramsList);
        RequestBody formBody = FormBody.create(MediaType.parse(Constants.CHARSET_JSON_UTF8), json);
        Map<String, String> headers = new HashMap<>();
        headers.put("REFERER", "app-referer");
        headers.put("USER-AGENT", "app-" + cn.com.qmth.examcloud.support.Constants.ELECTRON_EXAM_SHELL);
        return HttpUtils.doPost(requestUrl, formBody, key, token, headers);
    }

    @Override
    public Result submitPracticeExamRecord(String key, String token) throws Exception {
        //封装请求参数
        final String requestUrl = String.format("%s/api/ecs_oe_student/examControl/endExam", sysProperty.getApiDomain());
        return HttpUtils.doGet(requestUrl, key, token);
    }

    @Override
    public Result checkOnlineExamRecord(String key, String token) throws Exception {
        //封装请求参数
        final String requestUrl = String.format("%s/api/ecs_oe_student/examControl/checkExamInProgress", sysProperty.getApiDomain());
        Result<String> ret = HttpUtils.doGet(requestUrl, key, token);
        if (Constants.CODE_200.equals(ret.getCode()) && StringUtils.isNotBlank(ret.getData())) {
            JSONObject data = JSONObject.parseObject(ret.getData());
            if (Constants.COMMON_SUCCESS_CODE.equals(data.getString("code"))) {
                JSONObject bdata = data.getJSONObject("data");
                if (bdata != null) {
                    Long examId = bdata.getLong("examId");
                    if (examId != null) {
                        ExamSettingsCacheBean exam = CacheHelper.getExamSettings(examId);
                        if (ExamType.ONLINE.name().equals(exam.getExamType())) {
                            ExamPropertyCacheBean es = CacheHelper.getExamProperty(examId, ExamProperties.APP_EXAM_ENABLED.name());
                            if (!"true".equals(es.getValue())) {
                                ret.setCode(Constants.CODE_409);
                                ret.setDesc(Constants.DESC_409);
                            }
                        }
                    }
                }
            }
        }
        return ret;
    }

    @Override
    public Result getExamRecordPracticeHistoryList(String key, String token, String examStudentId) throws Exception {
        //封装请求参数
        final String requestUrl = String.format("%s/api/branch_ecs_oe_admin/practice/queryPracticeRecordList?examStudentId=%s", sysProperty.getApiDomain(), examStudentId);
        return HttpUtils.doGet(requestUrl, key, token);
    }

    @Override
    public Result getExamRecordTotalInfo(String key, String token, String examRecordId, String fromCache) throws Exception {
        //封装请求参数
        String requestUrl = null;
        if ("1".equals(fromCache)) {
            requestUrl = String.format("%s/api/branch_ecs_oe_admin/practice/getPracticeDetailInfo?fromCache=1&examRecordDataId=%s", sysProperty.getApiDomain(), examRecordId);
        } else {
            requestUrl = String.format("%s/api/branch_ecs_oe_admin/practice/getPracticeDetailInfo?examRecordDataId=%s", sysProperty.getApiDomain(), examRecordId);
        }
        return HttpUtils.doGet(requestUrl, key, token);
    }

    @Override
    public Result getExamRecordQuestionDetailList(String key, String token, String examRecordId) throws Exception {
        //封装请求参数
        final String requestUrl = String.format("%s/api/ecs_oe/report/exam_detail_report/paper_question?exam_record_id=%s", sysProperty.getApiDomain(), examRecordId);
        return HttpUtils.doGet(requestUrl, key, token);
    }

    @Override
    public Result getExamRecordQuestionAudioPlayTimes(String key, String token, String questionId) throws Exception {
        //封装请求参数
        final String requestUrl = String.format("%s/api/exam_question_playtimes?questionId=%s", sysProperty.getApiDomain(), questionId);
        return HttpUtils.doGet(requestUrl, key, token);
    }

    @Override
    public Result updateExamRecordQuestionAudioPlayTimes(String key, String token, String questionId, String mediaName) throws Exception {
        //封装请求参数
        final String requestUrl = String.format("%s/api/exam_question_playtimes", sysProperty.getApiDomain());
        RequestBody formBody = new FormBody.Builder()
                .add("questionId", questionId)
                .add("mediaName", mediaName)
                .build();
        return HttpUtils.doPost(requestUrl, formBody, key, token);
    }

    @Override
    public Result findExamRecordDataEntity(String key, String token, Long examRecordDataId, String fromCache) throws Exception {
        //封装请求参数
        String requestUrl = null;
        if ("1".equals(fromCache)) {
            requestUrl = String.format("%s/api/branch_ecs_oe_admin/exam/record/data/findExamRecordDataEntity?fromCache=1&examRecordDataId=%s", sysProperty.getApiDomain(), examRecordDataId);
        } else {
            requestUrl = String.format("%s/api/branch_ecs_oe_admin/exam/record/data/findExamRecordDataEntity?examRecordDataId=%s", sysProperty.getApiDomain(), examRecordDataId);
        }
        Result<String> result = HttpUtils.doGet(requestUrl, key, token);
        if (result.isSuccess()) {
            //过滤掉为空的属性
            result.setData(HttpUtils.filterNullAttributes(result.getData()));
        }
        return result;
    }

    @Override
    public Result getExamRecordQuestions(String key, String token, Long examRecordDataId, String fromCache) throws Exception {
        //封装请求参数
        String requestUrl = null;
        if ("1".equals(fromCache)) {
            requestUrl = String.format("%s/api/branch_ecs_oe_admin/examRecordQuestions/getExamRecordQuestions?fromCache=1&examRecordDataId=%s", sysProperty.getApiDomain(), examRecordDataId);
        } else {
            requestUrl = String.format("%s/api/branch_ecs_oe_admin/examRecordQuestions/getExamRecordQuestions?examRecordDataId=%s", sysProperty.getApiDomain(), examRecordDataId);
        }
        Result<String> result = HttpUtils.doGet(requestUrl, key, token);
        if (result.isSuccess()) {
            //过滤掉为空的属性
            result.setData(HttpUtils.filterNullAttributes(result.getData()));
        }
        return result;
    }

    @Override
    public Result getExamPropertyFromCacheByStudentSession(String key, String token, Long examId, String keys) throws Exception {
        //封装请求参数
        final String requestUrl = String.format("%s/api/ecs_exam_work/exam/getExamPropertyFromCacheByStudentSession/" + examId + "/" + keys, sysProperty.getApiDomain());
        Result<String> result = HttpUtils.doGet(requestUrl, key, token);
        return result;
    }

    @Override
    public Result getPaperById(String key, String token, String paperId) throws Exception {
        final String requestUrl = String.format("%s/api/branch_ecs_ques/paper/" + paperId, sysProperty.getApiDomain());
        Result<String> result = HttpUtils.doGet(requestUrl, key, token);
        return result;
    }

    @Override
    public Result getYunSignature(String key, String token, GetYunSignatureReq req) throws Exception {
        //封装请求参数
        final String requestUrl = String.format("%s/api/ecs_oe_student/examControl/yunSignature", sysProperty.getApiDomain());
        Builder b = new FormBody.Builder()
                .add("examRecordDataId", req.getExamRecordDataId())
                .add("order", req.getOrder())
                .add("fileMd5", req.getFileMd5())
                .add("fileSuffix", req.getFileSuffix());
        if (StringUtils.isNotBlank(req.getExt())) {
            b.add("ext", req.getExt());
        }
        RequestBody formBody = b.build();

        return HttpUtils.doPost(requestUrl, formBody, key, token);
    }

    @Override
    public Result queryExamList(String key, String token) throws Exception {
        final String requestUrl = String.format("%s/api/branch_ecs_oe_admin/examControl/queryExamList", sysProperty.getApiDomain());
        return HttpUtils.doGet(requestUrl, key, token);
    }

    @Override
    public Result getEndExamInfo(String key, String token, Long examRecordDataId) throws Exception {
        final String requestUrl = String.format("%s/api/ecs_oe_student/examControl/getEndExamInfo?examRecordDataId=" + examRecordDataId, sysProperty.getApiDomain());
        return HttpUtils.doGet(requestUrl, key, token);
    }

    @Override
    public Result queryObjectiveScoreList(String key, String token, Long examStudentId) throws Exception {
        final String requestUrl = String.format("%s/api/branch_ecs_oe_admin/exam/score/queryObjectiveScoreList?examStudentId=" + examStudentId, sysProperty.getApiDomain());
        return HttpUtils.doGet(requestUrl, key, token);
    }

    @Override
    public Result startAnswer(String key, String token, Long examRecordDataId) throws Exception {
        final String requestUrl = String.format("%s/api/ecs_oe_student/examControl/startAnswer", sysProperty.getApiDomain());
        RequestBody formBody = new FormBody.Builder()
                .add("examRecordDataId", examRecordDataId.toString())
                .build();
        return HttpUtils.doPost(requestUrl, formBody, key, token);
    }

    @Override
    public Result uploadPaperAnswerBatch(String key, String token, Long examRecordDataId, MultipartFile[] fileArray,
                                         String fileType, String[] fileMd5Array) throws Exception {
        if (!Constants.FILE_TYPE_ZIP.equals(fileType) && !Constants.FILE_TYPE_PDF.equals(fileType)
                && !Constants.FILE_TYPE_JPG.equals(fileType) && !Constants.FILE_TYPE_JPEG.equals(fileType)
                && !Constants.FILE_TYPE_PNG.equals(fileType)) {
            throw new ApiException("FileType must be zip,pdf,jpg,jpeg,png.");
        }
        //封装请求参数									
        final String requestUrl = String.format("%s/api/branch_ecs_oe_admin/offlineExam/batchSubmitPaper?examRecordDataId=" + examRecordDataId + "&fileType=" + fileType, sysProperty.getApiDomain());
        MultipartBody.Builder form = new MultipartBody.Builder().setType(MultipartBody.FORM);
        for (MultipartFile file : fileArray) {
            RequestBody body = RequestBody.create(MediaType.parse("application/octet-stream"), file.getBytes());
            form.addFormDataPart("fileArray", file.getOriginalFilename(), body);
        }
        form.addFormDataPart("fileType", fileType);
        for (String md5 : fileMd5Array) {
            form.addFormDataPart("fileMd5Array", md5 != null ? md5 : "");
        }

        return HttpUtils.doPost(requestUrl, form.build(), key, token);
    }

}