package cn.com.qmth.examcloud.app.service.impl;

import cn.com.qmth.examcloud.app.core.SysProperty;
import cn.com.qmth.examcloud.app.core.utils.HttpUtils;
import cn.com.qmth.examcloud.app.model.CaptureInfo;
import cn.com.qmth.examcloud.app.model.Constants;
import cn.com.qmth.examcloud.app.model.Result;
import cn.com.qmth.examcloud.app.service.FaceVerifyService;
import cn.com.qmth.examcloud.commons.util.JsonMapper;
import okhttp3.FormBody;
import okhttp3.MediaType;
import okhttp3.RequestBody;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

/**
 * 人脸识别、比对验证相关接口
 */
@Service
public class FaceVerifyServiceImpl implements FaceVerifyService {

    private static final Logger log = LoggerFactory.getLogger(FaceVerifyServiceImpl.class);

    @Autowired
    private SysProperty sysProperty;

    @Override
    public Result faceCheckEnabled(String key, String token, Long examId) throws Exception {
        final String requestUrl = String.format("%s/api/ecs_exam_work/exam/faceCheckEnabled/%s", sysProperty.getApiDomain(), examId);
        return HttpUtils.doGet(requestUrl, key, token);
    }

    @Override
    public Result getPhotoSign(String key, String token, String fileSuffix, String fileMd5) throws Exception {
        final String requestUrl = String.format("%s/api/ecs_oe_student/examControl/getCapturePhotoYunSign?fileSuffix=%s&fileMd5=%s",
                sysProperty.getApiDomain(), fileSuffix != null ? fileSuffix : "", fileMd5 != null ? fileMd5 : "");

        return HttpUtils.doGet(requestUrl, key, token);
    }

    @Override
    public Result compare(String key, String token, String fileUrl, String signIdentifier) throws Exception {
        final String requestUrl = String.format("%s/api/ecs_oe_student/examCaptureQueue/compareFaceSync?fileUrl=%s&signIdentifier=%s",
                sysProperty.getApiDomain(), fileUrl != null ? fileUrl : "", signIdentifier != null ? signIdentifier : "");

        RequestBody formBody = new FormBody.Builder().build();
        return HttpUtils.doPost(requestUrl, formBody, key, token);
    }

    @Override
    public Result saveCaptureInfo(String key, String token, CaptureInfo captureInfo) throws Exception {
        final String requestUrl = String.format("%s/api/ecs_oe_student/examCaptureQueue/uploadExamCapture", sysProperty.getApiDomain());

        String json = new JsonMapper().toJson(captureInfo);
        RequestBody formBody = FormBody.create(MediaType.parse(Constants.CHARSET_JSON_UTF8), json);
        return HttpUtils.doPost(requestUrl, formBody, key, token);
    }

    @Override
    public Result getCaptureResult(String key, String token, Long examRecordDataId, String fileName) throws Exception {
        final String requestUrl = String.format("%s/api/ecs_oe_student/examCaptureQueue/getExamCaptureResult?examRecordDataId=%s&fileName=%s",
                sysProperty.getApiDomain(), examRecordDataId, fileName != null ? fileName : "");

        return HttpUtils.doGet(requestUrl, key, token);
    }

}
